package com.esd.ucm.util;

/**
 * @author SLiu
 *
 */
import java.util.*;
import java.util.logging.*;
import java.io.*;

public final class Cusip2Isin 
{
	private Logger logger = Logger.getLogger("Cusip2Isin");
	
	private final String PROPERTY_FILE = "cusip2isin.properties";
	private final String C2I_MAP_FILE = "MellonSecurities.xls";
	private final String FILES = "FILES";
	private final String OUT_FILES = "OUT_FILES";
	private final String  CUSIP_INDEX = "CUSIP_INDEX";
	private final String EXCEL_MAP_FILE = "EXCEL_MAP_FILE";
	private final String HEADER_ROWS = "HEADER_ROWS";
	
	
	
	private Properties config = null;
	private Cusip2IsinMap map = null;
	private int[] cusip_indices = null;
	private String map_file = C2I_MAP_FILE;
	private String[] srcList = null;
	private String[] destList = null;
	private int[] headerRows = null;
	
	private String propertyFile = null;
	
	/**
	 * Constructor for Cusip2Isin.
	 */
	public Cusip2Isin() 
	{
		super();		
	}
	
	protected void init()
	{
		InputStream in = null;
		try{
			//if properties file has been specified explicitly, we will use it,
			//Otherwise 
			if (propertyFile != null) in = new FileInputStream(propertyFile);
			if (in==null) in = this.getClass().getResourceAsStream("/cusip2isin.properties");
			if (in==null) in = this.getClass().getResourceAsStream("cusip2isin.properties");
			if (in==null) throw new Exception("cusip2isin.properties file not found.");
			config = new Properties();
			config.load(in);
			
			if ( config.getProperty(FILES)!=null) 
						srcList = config.getProperty(FILES).split(",");
			
			if ( config.getProperty(OUT_FILES)!=null) 
						destList = config.getProperty(OUT_FILES).split(",");
						
			if ( config.getProperty(EXCEL_MAP_FILE)!=null) 
						map_file = config.getProperty(EXCEL_MAP_FILE);	
			
			headerRows = new int[srcList.length];			
			
			if ( config.getProperty(CUSIP_INDEX)!=null){
						 String[] idxs = config.getProperty(CUSIP_INDEX).split(",");
						 cusip_indices = new int[idxs.length];
						 for(int i=0; i<idxs.length; i++)
						 {
						 	cusip_indices[i] = Integer.valueOf(idxs[i]).intValue();
						 }
						 
				}
			if ( config.getProperty(HEADER_ROWS)!=null){
				 String[] idxs = config.getProperty(HEADER_ROWS).split(",");				 
				 for(int i=0; i<idxs.length; i++)
				 {
				 	headerRows[i] = Integer.valueOf(idxs[i]).intValue();
				 }					 
			}	
				
			map =  new Cusip2IsinMap();
			map.loadFromXls(map_file);			 		
					
		}catch(Exception ex){
			throw new RuntimeException(ex);	
		}finally{
			if (in!=null) try{ in.close();}catch(Exception ex){;}
		}
	}
	
	
	public void runMapping()
	{
		init();
		int length = srcList.length;
		for(int i=0; i<length; i++)
		{
			map(srcList[i], destList[i], cusip_indices[i], headerRows[i]);			
		}		
	}	
	
	protected void map(String in, String out, int idx, int headerRow)
	{
		BufferedReader reader = null;
		PrintWriter writer = null;
		try{
			reader = new BufferedReader(new FileReader(in));
			writer = new PrintWriter(new FileWriter(out));
			String line = null;
			
			for(int i=0; i<headerRow; i++){
				line = reader.readLine();
				CSVRowUtil header = new CSVRowUtil(line, ",");				
				writer.println(header.toString());
			}
									
			while((line=reader.readLine())!=null)
			{
				CSVRowUtil data = new CSVRowUtil(line, ",");
				String key = data.getToken(idx).trim();
				String isin = map.getProperty(key);
				if (isin!=null){
					data.replace(idx, isin);
				}else{
					logger.log(Level.WARNING, "No mapping found for this Cusip Number [" + key + "] on [" + in + "]");
				}
				
				
				writer.println(data.toString());
			}			
			
		}catch(Exception ex){
			logger.log(Level.WARNING, "Error at process this file [" +in + "]", ex);
		}finally{
			if (reader!=null) try{ reader.close();}catch(Exception ex1){;}
			if (writer!=null) try{ writer.close();}catch(Exception ex2){;}
		}		
		
	}
	
	

	public static void main(String[] args) 
	{
		
		Cusip2Isin mapper = new Cusip2Isin();
		if (args.length>0) {			
			mapper.setPropertyFile(args[0]);
		}
		mapper.runMapping();
	}
	
	
	/**
	 * Returns the propertyFile.
	 * @return String
	 */
	public String getPropertyFile() {
		return propertyFile;
	}

	/**
	 * Sets the propertyFile.
	 * @param propertyFile The propertyFile to set
	 */
	public void setPropertyFile(String propertyFile) {
		System.out.println(propertyFile);
		this.propertyFile = propertyFile;
	}

}
