package com.esd.util;

import java.sql.*;
import java.util.*;

/*
 * SQLStatementConstructor is a utility class that will substitute '?' characters
 * in a SQL query with arguments provided to a setXXX method.
 *
 * Created on August 29, 2002, 1:51 PM
 * @author  SLiu
 */
public class SQLStatementConstructor {

    protected String[] parts;
    protected int[] offsets;

    /////////////////////////////////////////////////////////////////////////////////
    // CONSTRUCTOR
    /////////////////////////////////////////////////////////////////////////////////

    /** Creates a new instance of SQLStatementConstructor */
    public SQLStatementConstructor(String pattern) {
        parse(pattern);
    }


    /////////////////////////////////////////////////////////////////////////////////
    // PUBLIC INTERFACE
    /////////////////////////////////////////////////////////////////////////////////

    /**
     * Update the statement to substitute the value for the argNumberth '?'
     * with appropriate SQL quoting and formatting
     *
     * @param argNumber the number of the '?' to substitute, starting from 1
     * @param value the new value
     * @throws SQLException if argNumber is not valid
     */
    public void setDouble(int argNumber, double value) throws SQLException {
        setNoQuote(argNumber, String.valueOf(value));
    }

    /**
     * Update the statement to substitute the value for the argNumberth '?'
     * with appropriate SQL quoting and formatting
     *
     * @param argNumber the number of the '?' to substitute, starting from 1
     * @param value the new value
     * @throws SQLException if argNumber is not valid
     */
    public void setFloat(int argNumber, float value) throws SQLException {
        setNoQuote(argNumber, String.valueOf(value));
    }

    /**
     * Update the statement to substitute NULL for the argNumberth '?'
     * with appropriate SQL quoting and formatting
     *
     * @param argNumber the number of the '?' to substitute, starting from 1
     * @throws SQLException if argNumber is not valid
     */
    public void setNull(int argNumber) throws SQLException {
        setNoQuote(argNumber, "NULL");
    }

    /**
     * Update the statement to substitute the value for the argNumberth '?'
     * with appropriate SQL quoting and formatting
     *
     * @param argNumber the number of the '?' to substitute, starting from 1
     * @param value the new value
     * @throws SQLException if argNumber is not valid
     */
    public void setInt(int argNumber, int value) throws SQLException {
        setNoQuote(argNumber, String.valueOf(value));
    }

    /**
     * Update the statement to substitute the value for the argNumberth '?'
     * with appropriate SQL quoting and formatting
     *
     * @param argNumber the number of the '?' to substitute, starting from 1
     * @param value the new value
     * @throws SQLException if argNumber is not valid
     */
    public void setLong(int argNumber, long value) throws SQLException {
        setNoQuote(argNumber, String.valueOf(value));
    }

    /**
     * Update the statement to substitute the value for the argNumberth '?'
     * with appropriate SQL quoting and formatting
     *
     * @param argNumber the number of the '?' to substitute, starting from 1
     * @param value the new value
     * @throws SQLException if argNumber is not valid
     */
    public void setChar(int argNumber, char value) throws SQLException {
        setNoQuote(argNumber, String.valueOf(value));
    }

    /**
     * Update the statement to substitute the value for the argNumberth '?'
     * with appropriate SQL quoting and formatting
     *
     * @param argNumber the number of the '?' to substitute, starting from 1
     * @param date the new value
     * @throws SQLException if argNumber is not valid
     */
    public void setDate(int argNumber, java.sql.Date date) throws SQLException {
        setString(argNumber, date.toString());
    }

    /**
     * Update the statement to substitute the value for the argNumberth '?'
     * with appropriate SQL quoting and formatting
     *
     * @param argNumber the number of the '?' to substitute, starting from 1
     * @param time the new value
     * @throws SQLException if argNumber is not valid
     */
    public void setTime(int argNumber, java.sql.Time time) throws SQLException {
        setString(argNumber, time.toString());
    }

    /**
     * Update the statement to substitute the value for the argNumberth '?'
     * with appropriate SQL quoting and formatting
     *
     * @param argNumber the number of the '?' to substitute, starting from 1
     * @param time the new value
     * @throws SQLException if argNumber is not valid
     */
    public void setTimestamp(int argNumber, java.sql.Timestamp time) throws SQLException {
        setString(argNumber, time.toString());
    }

    /**
     * Update the statement to substitute the value for the argNumberth '?'
     * with appropriate SQL quoting and formatting
     *
     * @param argNumber the number of the '?' to substitute, starting from 1
     * @param columnName the new value
     * @throws SQLException if argNumber is not valid
     */
    public void setColumnName(int argNumber, String columnName) throws SQLException {
        setNoQuote(argNumber, columnName);
    }

    /**
     * Update the statement to substitute the value for the argNumberth '?'
     * with appropriate SQL quoting and formatting
     *
     * @param argNumber the number of the '?' to substitute, starting from 1
     * @param tableName the new value
     * @throws SQLException if argNumber is not valid
     */
    public void setTableName(int argNumber, String tableName) throws SQLException {
        setNoQuote(argNumber, tableName);
    }

    /**
     * Update the statement to substitute the value for the argNumberth '?'
     * with appropriate SQL quoting and formatting
     *
     * @param argNumber the number of the '?' to substitute, starting from 1
     * @param value the new value
     * @throws SQLException if argNumber is not valid
     */
    public void setNoQuote(int argNumber, String value) throws SQLException {
        if (argNumber > offsets.length || argNumber <= 0) {
            throw new SQLException("Parameter index out of bound exception.");
        }
        parts[offsets[argNumber - 1]] = value;

    }

    /**
     * Update the statement to substitute the value for the argNumberth '?'
     * with appropriate SQL quoting and formatting
     *
     * @param argNumber the number of the '?' to substitute, starting from 1
     * @param value the new value
     * @throws SQLException if argNumber is not valid
     */
    public void setString(int argNumber, String value) throws SQLException {
        value = "'" + value + "'";
        setNoQuote(argNumber, value);
    }

    /**
     * Return the SQL statement constructed.
     */
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        for (int i = 0; i < parts.length; i++) {
            buffer.append(parts[i]);
        }
        return buffer.toString();
    }

    public static void main(String arg[]) throws Exception {
        String test = "SELECT ?, ?, name FROM ? WHERE ?=?";
        SQLStatementConstructor sql = new SQLStatementConstructor(test);
        sql.setColumnName(1, "id");
        sql.setColumnName(2, "type");
        sql.setTableName(3, "userlist");
        sql.setColumnName(4, "password");
        sql.setString(5, "XYZ");
        System.out.println(sql.toString());
        String firstWord = (sql.toString()).substring(0, (sql.toString()).indexOf(" "));
        System.out.println(firstWord);
    }

    /////////////////////////////////////////////////////////////////////////////////
    // IMPLEMENTATION
    /////////////////////////////////////////////////////////////////////////////////

    protected void parse(String pattern) {
        StringBuffer sb = new StringBuffer();
        Vector v = new Vector(30);
        Vector offsetsV = new Vector(30);
        boolean inString = false;
        int count = 0;
        for (int i = 0; i < pattern.length(); i++) {
            char c = pattern.charAt(i);
            if (c == '\'' && !inString) {
                sb.append(c);
                inString = true;
            } else if (c == '\'' && inString) {
                sb.append(c);
                inString = false;
            } else if (c == '?' && !inString) {
                v.add(sb.toString());
                count++;
                offsetsV.add(new Integer(count));
                v.add("?");
                count++;
                sb.setLength(0);
            } else {
                sb.append(c);
            }
        }
        v.add(sb.toString());

        parts = new String[v.size()];
        v.copyInto(parts);
        offsets = new int[offsetsV.size()];
        Enumeration enum = offsetsV.elements();
        count = 0;
        while (enum.hasMoreElements()) {
            Integer value = (Integer) enum.nextElement();
            offsets[count++] = value.intValue();
        }
    }

}
