package com.esdi.sql;

/**
 * <p>Title: CITCOReferenceProvider </p>
 * <p>Description: CITCO's Reference Provider Class</p>
 * @author Anatoly Kagan
 * @version 1.0
 */


import java.sql.*;
import java.util.*;
import java.net.*;

public class ESDConnectionManager {

    private static Connection conn = null;

    private static int retryCounter = 0;
    private static final int MAX_RETRIES = 3;
    private static final int SLEEP_RETRY = 1000;       // ms

    /**
     * system properties are expected to be populated with appropriate values
     * @return connection
     */
    public static Connection getConnection() throws SQLException {

        if (conn != null) {

            if (conn.isClosed()) {

                while (retryCounter < MAX_RETRIES) {

                    retryCounter++;
                    System.out.println("Trying to reestablish connection. Attempt " + retryCounter);
                    // this will cause an attempt to reestablish connection
                    conn = null;
                    // making recursive call
                    try {
                        conn = getConnection();
                        if (conn != null && !conn.isClosed()) {
                            System.out.println("Connection reestablished succesfully on " + retryCounter + " attempt");
                            retryCounter = 0;
                            break;
                        }
                    } catch (SQLException e) {
                        System.err.println("Failed to get Connection " + e.getMessage());
                        if (retryCounter >= MAX_RETRIES) {
                            throw new SQLException("Unable to establish connection after " +
                                MAX_RETRIES + "  attempts");
                        } else {
                            // sleep before trying again
                            try {
                                Thread.sleep(SLEEP_RETRY);
                            } catch (InterruptedException e2) {}
                        }
                    }
                }
            }
            return conn;
        }
        String serverName = System.getProperty("db.server");
        String driver = System.getProperty("ESD.JDBC.driver");
        String url = System.getProperty("ESD.DbServer." + serverName + ".url");
        String user= System.getProperty("db.uname");
        String password=System.getProperty("db.pswd");

        Properties props = new Properties();
        props.put("user", user);
        props.put("password", password);
        props.put("IGNORE_DONE_IN_PROC", "true");

        String hostName;
        try {
          hostName = java.net.InetAddress.getLocalHost().getHostName();
        } catch (Exception e) {
          hostName = "unknown";
        }
        props.put("hostname", hostName);

        //System.out.println("Url: " + url);
        //System.out.println("Hostname: " + hostName);
        //System.out.println("User: " + props.get("user"));
        //System.out.println("Password: " + props.get("password"));
        //System.out.println("Driver: " + driver);

        try {
            Class.forName(driver);
            conn = DriverManager.getConnection(url, props);
        } catch (ClassNotFoundException e) {
            System.err.println("ClassNotFoundException: " + e.getMessage());
            e.printStackTrace();
            throw new SQLException(e.getMessage());
        }
        return conn;
    }

    /**
     * @clienName is name of client from properties file.
     * @return a open connection for client databases, Caller is resposible for closing it
     */
    public static Connection getClientConnection(String clientName) throws SQLException {
        Connection clientConn;
        String serverName = clientName.trim();
        String driver = System.getProperty("ESD.JDBC.driver");
        String url = System.getProperty("ESD.DbServer." + serverName + ".url");
        String user= System.getProperty(serverName + ".db.uname");
        String password=System.getProperty(serverName + ".db.pswd");

        Properties props = new Properties();
        props.put("user", user);
        props.put("password", password);
        props.put("IGNORE_DONE_IN_PROC", "true");

        String hostName;
        try {
          hostName = java.net.InetAddress.getLocalHost().getHostName();
        } catch (Exception e) {
          hostName = "unknown";
        }
        props.put("hostname", hostName);

        //System.out.println("Url: " + url);
        //System.out.println("Hostname: " + hostName);
        //System.out.println("User: " + props.get("user"));
        //System.out.println("Password: " + props.get("password"));
        //System.out.println("Driver: " + driver);

        try {
            Class.forName(driver);
            clientConn = DriverManager.getConnection(url, props);
        } catch (ClassNotFoundException e) {
            System.err.println("ClassNotFoundException: " + e.getMessage());
            e.printStackTrace();
            throw new SQLException(e.getMessage());
        }
        // connection was established resetting the retry counter
        retryCounter = 0;
        return clientConn;
    }

    /**
     * Releases a connection back to this manager. In the current implementation
     * does nothing. In the future can interface with an application server for
     * connection pooling.
     */
    public static void releaseConnection(Connection conn) {
        if (conn != null) {
            // do nothing
        }
    }

    public static void closeConnection() {
        try {
            if (conn != null) conn.close();
        } catch (SQLException e) {
            System.err.println("SQLException: Cannot close connection " + e.getMessage());
            e.printStackTrace();
        }
        conn = null;
    }

}

