package com.esdi.test;

import java.sql.*;
import java.util.Map;
import java.text.SimpleDateFormat;

/**
 A utility class to help with testing.  It provides a simplified API
 for getting data from the database or performing queries.
 <p>
 To use a TestQuery
 1. create an instance passing inthe SQL string -- this will execute the query
 (which may be a select, update, insert or delete).
 2. If the query returns a result,	the TestQuery sets its result set to the first row
 so that callers can use the getXXX methods to look at values.
 */
public class TestQuery {
    public static final int TRUE = 1;

    protected Connection dbConnection;
    protected ResultSet results = null;
    protected int currentRow = -1;
    protected int rowsAffected = -1;

    protected String url, user, query;

    /**
     Creates a TestQuery and executes the query immediately.  The results are
     loaded and can be interrogated.  The query may be any valid SQL string.
     <p>
     It is assumed that the standard ESD properties have been set up

     @param name -- the name of the database in the sybase interfaces file and application.properties file
     @param aQuery -- the sql string to be executed
     */
    public TestQuery(String name, String aQuery)
        throws Exception {
        connect(name, null);
        query = aQuery;
        executeQuery();
    }

    /**
     Creates a TestQuery and executes the query immediately.  The results are
     loaded and can be interrogated.  The query may be any valid SQL string.
     <p>
     It is assumed that the standard ESD properties have been set up

     @param name -- the name of the database in the sybase interfaces file and application.properties file
     @param productionPassword -- the password for the system.  May be a production server
     @param aQuery -- the sql string to be executed
     */
    public TestQuery(String name, String productionPassword, String aQuery)
        throws Exception {
        connect(name, productionPassword);
        query = aQuery;
        executeQuery();
    }

    /**
     Creates a TestQuery and executes the query immediately.  The results are
     loaded and can be interrogated.  The query may be any valid SQL string.
     <p>
     It is assumed that the standard ESD properties have been set up

     @param name -- the name of the database in the sybase interfaces file and application.properties file
     @param aQuery -- the sql string to be executed
     @param ignoreErrors -- ignored, but should be true
     */
    public TestQuery(String name, String aQuery, boolean ignoreErrors) {
        super();
        try {
            connect(name, null);
            query = aQuery;
            executeQuery();
        } catch (Exception ex) {
            System.out.println("Ignoring error from executeQuery [" + query + "]" + ex);
        }
    }


    protected void connect(String name, String productionPassword)
        throws Exception {
        String driverClassName = System.getProperty("ESD.JDBC.driver");
        try {
            Class.forName(driverClassName);
        } catch (ClassNotFoundException e) {
            throw new RuntimeException("Could not load database driver: " + driverClassName);
        }
        url = System.getProperty("ESD.DbServer." + name + ".url");
        user = System.getProperty(name.equalsIgnoreCase("global") ? "db.uname" : name + ".db.uname");
        String password = null;
        if (productionPassword == null) {
            password = System.getProperty(name.equalsIgnoreCase("global") ? "db.pswd" : name + ".db.pswd");
            if (url.indexOf("128.127.36.15") == -1) {
                throw new Exception("Cannot connect to a non-development server " + url + " for testing");
            }
        } else {
            password = productionPassword;
        }
        dbConnection = DriverManager.getConnection(url, user, password);
    }


    /**
     Disconnect from the database by closing the connection
     */
    public void disconnect() throws SQLException {
        dbConnection.close();
    }


    /**
     Returns the number of rows in the result set.
     This sets the result set to the first record, losing any
     place in the results.
     */
    public int count() {
        try {
            int count = results.getRow();
            while (results.next()) {
                count++;
            }
            first();
            return count;
        } catch (SQLException ex) {
            System.err.println(ex);
            return 0;
        }
    }


    /**
     Remove the database collection when this object
     is finalized by the garbage collector
     */
    public void finalize() {
        try {
            results.close();
        } catch (Exception ex) {
            // do nothing
        }
        try {
            dbConnection.close();
        } catch (Throwable t) {
            // do nothing
        }
    }


    /**
     Set the current row to the first row
     */
    public void first()
        throws SQLException {
        boolean thereAreRows = results.first();
        if (thereAreRows) {
            currentRow = 1;
        }
    }


    /**
     Returns the value in the current record of the field, or Null
     if the column is not of the right type to return an Object
     By default this will be the first record unless next()
     has been called
     */
    public Object get(String field)
        throws SQLException {
        return results.getObject(field);
    }


    /**
     Returns the value in the current record of the field.
     By default this will be the first record unless next()
     has been called
     */
    public boolean getBoolean(String field)
        throws SQLException {
        return results.getBoolean(field);
    }


    /**
     Returns the SQL command being executed, eg "select"
     */
    protected String getCommand(String aQuery) {
        int pos = 0;
        String trimmedQuery = aQuery.trim();
        while (pos < trimmedQuery.length()
            && " \n\t\r".indexOf(trimmedQuery.charAt(pos)) == -1) {
            pos++;
        }
        return trimmedQuery.substring(0, pos);
    }


    /**
     Returns the number of the current row, starting from 1.
     Returns -1 if there are no valid rows or if the query was
     and update, delete, or insert.
     */
    public int getCurrentRow() {
        return currentRow;
    }


    /**
     Returns the value in the current record of the field.
     By default this will be the first record unless next()
     has been called
     */
    public java.sql.Date getDate(String field)
        throws SQLException {
        return results.getDate(field);
    }


    /**
     Returns the value in the current record of the field.
     By default this will be the first record unless next()
     has been called.
     */
    public int getInt(String field)
        throws SQLException {
        return results.getInt(field);
    }


    /**
     Returns the value in the current record of the field.
     By default this will be the first record unless next()
     has been called.
     */
    public double getDouble(String field)
        throws SQLException {
        return results.getDouble(field);
    }


    /**
     Returns the resultSet for the query
     */
    public java.sql.ResultSet getResults() {
        return results;
    }


    /**
     Returns the number of rows affected by the query.  If the
     query was a select, this will be equal to count().  If the query
     was an insert, update or delete, it will be the number of rows
     affected by that query.
     */
    public int getRowsAffected() {
        if (rowsAffected == -1) {
            return count();
        } else {
            return rowsAffected;
        }
    }


    /**
     Returns the value in the current record of the field.
     By default this will be the first record unless next()
     has been called
     */
    public Timestamp getTimestamp(String field)
        throws SQLException {
        return results.getTimestamp(field);
    }


    /**
     * Returns true if the result set has another row
     */
    public boolean hasNext()
    throws SQLException {
        return ! results.isLast();
    }
    /**
     Returns true if there is another row and
     advances to that row, updating the currentRow
     */
    public boolean next()
        throws SQLException {
        boolean wasAdvanced = results.next();
        if (wasAdvanced) {
            currentRow++;
        }
        return wasAdvanced;
    }


    /**
     Returns true if the value in the last field accessed in the current record
     was null.  I know, I know, I don't like this state-based behaviour either, but it's
     all ResultSet's fault.
     By default this will be the first record unless next() has been called.
     */
    public boolean wasNull()
        throws SQLException {
        return results.wasNull();
    }

    /**
     Returns the value in the current record of the field.
     By default this will be the first record unless next()
     has been called
     */
    public String getString(String field)
        throws SQLException {
        return results.getString(field);
    }

    /**
     Returns the value in the current record of the field.
     By default this will be the first record unless next()
     has been called. Counts from 1
     */
    public Object getColumn(int column)
        throws SQLException {
        return results.getObject(column);
    }

    public static void main(String[] args) {
        try {
            com.esdi.init.ESDConfig.loadApplicationProperties();
            TestQuery t = new TestQuery("global", "select count(*) c from common..instrument_type");
            System.out.println("Result = " + t.getInt("c"));
        } catch (Throwable tt) {
            System.err.println(tt);
        }
    }

    /**
     * Do you hate writing insert statements? I do.  That's why I wrote a method that would write one for me.
     * This builds an insert statement modelled on the results for this TestQuery instance.
     *
     * @param table -- name of the database.ownwer.table for the insert
     * @param substituteValues Map of column->value (String) to substitute based on the data in the current TestQuery
     * @return an insert statement that creates a row containing the current result set with substitutions
     * @throws java.sql.SQLException
     */
    public String asInsertStatement(String table, Map substituteValues)
        throws SQLException {
        StringBuffer columns = new StringBuffer(" (");
        StringBuffer values = new StringBuffer(" values (");
        ResultSetMetaData meta = results.getMetaData();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd hh:mm:ss.SSS");
        for (int c = 1; c <= meta.getColumnCount(); c++) {  // 1 - based
            String column = meta.getColumnName(c);
            columns
                .append(column)
                .append(c < meta.getColumnCount() ? ", " : ")\n");
            String value = (String) substituteValues.get(column);
            if (value == null) {
                Object dbValue = results.getObject(c);
                if (results.wasNull()) {
                    value = "NULL";
                } else {
                    int type = meta.getColumnType(c);
                    switch (type) {
                        case Types.CHAR:
                        case Types.VARCHAR:
                        case Types.LONGVARCHAR:
                            value = "'" + dbValue.toString().trim() + "'";
                            break;
                        case Types.DATE:
                        case Types.TIME:
                        case Types.TIMESTAMP:
                            value = "'" + sdf.format(results.getDate(meta.getColumnName(c))) + "'";
                            break;
                        default :
                            value = "" + dbValue.toString();
                            break;
                    }
                }
            }
            values
                .append(value)
                .append(c < meta.getColumnCount() ? ", " : ")");
        }
        return "insert " + table + "\n"
            + columns.toString()
            + values.toString();
    }

    /**
     * Do you hate writing insert statements? I do.  That's why I wrote a method that would write one for me.
     * This builds an insert statement modelled on the results for this TestQuery instance.
     *
     * @param table -- name of the database.ownwer.table for the insert
     * @param substituteValues Map of column->value (String) to substitute based on the data in the current TestQuery
     * @return an insert statement that creates a row containing the current result set with substitutions
     * @throws java.sql.SQLException
     */
    public String asInsertStatements(String table, Map substituteValues)
        throws SQLException {
        StringBuffer inserts = new StringBuffer();
        int count = count();    // count() resets cursor, so no good in for() condition
        for (int r = 0; r < count; r++) {
            inserts
                .append(asInsertStatement(table, substituteValues))
                .append("\n");
            next();
        }
        return inserts.toString();
    }

    /**
     * Compare the resultset with the same query run in dbInstance.  The query should return
     * records in the same order in both databases.
     *
     * @param dbInstance the Sybase name of the database to compare with
     * @return true if all records are equal in both dbInstances
     * @throws Exception
     */
    public boolean same(String dbInstance)
    throws Exception {
        TestQuery q = new TestQuery(dbInstance, query);
        int i = 0;
        if (count() == q.count()) {
//            System.out.print(i++ + ": ");
            ResultSetMetaData meta = results.getMetaData();
            ResultSetMetaData qMeta = q.results.getMetaData();
            if (meta.getColumnCount() != qMeta.getColumnCount()) {
                return false;
            }
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd hh:mm:ss.SSS");
            do {
                for (int c = 1; c <= meta.getColumnCount(); c++) {  // 1 - based
                    Object value = results.getObject(c);
                    Object qValue = q.results.getObject(c);
//                    System.out.print(value + "=" + qValue);
                    boolean columnSame = false;
                        if (results.wasNull() || q.results.wasNull()) {
                            if (results.wasNull() && q.results.wasNull()) {
                                columnSame = true;
                            } else {
                                return false;
                            }
                        } else {
                            int type = meta.getColumnType(c);
                            switch (type) {
                                case Types.DATE:
                                case Types.TIME:
                                case Types.TIMESTAMP:
                                    columnSame = sdf.format(value).equals(sdf.format(qValue));
                                    break;
                                default :
                                    columnSame = value.equals(qValue);
                            }
                        }
//                    System.out.print(columnSame ? ", " : " X");
                    if (!columnSame) {
                        System.out.println("*** Row " + i + ": " + meta.getColumnName(c) + " different");
                        return false;
                    }
                }
//                System.out.println();
                q.next();
            } while (next());
            return true;
        } else {
            return false;
        }
    }

    /////////////////////////////////////////////////////////////////////////////////
    // IMPLEMENTATION
    /////////////////////////////////////////////////////////////////////////////////

    /**
     Execute the query passed in as a String, allowing rows
     to be fetched in a forwards and backwards direction.
     Returns the results if the query was a select, or null if it was
     an update, insert, or delete
     */
    protected ResultSet executeQuery()
        throws SQLException {
        try {
            if (echoQuery()) {
                System.out.println("[" + url + ", " + user + "]\n " + query);
            }
            java.sql.Statement sqlStmt = dbConnection.createStatement(
                ResultSet.TYPE_SCROLL_SENSITIVE,
                ResultSet.CONCUR_READ_ONLY);
            if (getCommand(query).equalsIgnoreCase("select") && query.indexOf("into") == -1) {
                results = sqlStmt.executeQuery(query);
                first();
                return results;
            } else {
                rowsAffected = sqlStmt.executeUpdate(query);
                return null;
            }
        } catch (SQLException x) {
            System.err.println("Error in executeQuery [" + query + "]" + x);
            throw x;
        }
    }

    protected boolean echoQuery() {
        return new Boolean(System.getProperty("test_query.echo", "false")).booleanValue();
    }


}
