package com.esdi.util;

/**
 * <p>Title: CUSIP checkdigit utility</p>
 * <p>Description: Class to generate and/or check if validity of
 * checkdigit</p>
 * <p>Copyright: Copyright (c) goNetix 2002</p>
 * <p>Company: goNetix </p>
 * @author Joe LeVasseur Joe@LeVasseur.com
 * @version 1.0
 */

import java.util.Hashtable;
import java.lang.Math.*;

public class CUSIP
{

    private CUSIP() // Private constructor use static- avoids making
    {               // the lookup array umpteen million times.
    }
    //--------------------------------------------------------------------------
    private static final char[] LEGAL_VALUES = {'0','1','2','3','4','5','6',
        '7','8','9','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O',
        'P','Q','R','S','T','U','V','W','X','Y','Z','*','@','#'};

    private static final int STD_CUSIP_LENGTH = 9;
    //--------------------------------------------------------------------------

    public static String addChkDigit(String cusip)
    {
        String szRet = cusip;
        String szCusip = null;
        String szCalculatedCheckDigit = null;
        int cusipLen = 0;
        String szCurrentCheckDigit = null;
        try
        {
            cusipLen = cusip.length();
            switch(cusipLen)
            {
                case 9:
                    szCusip = cusip.substring(0,8);
                    szCurrentCheckDigit = cusip.substring(8,9);
                    break;
                case 7:
                    szCusip = cusip.substring(0,6);
                    szCurrentCheckDigit = cusip.substring(6,7);
                    break;
                case 8:
                case 6:
                    szCusip = cusip;
                    break;
                default:
                    throw new Exception("CUSIP must be either 6 or 8 chars before checkdigit");
            }
            szCalculatedCheckDigit = generateCheckDigit(szCusip);
            if(szCurrentCheckDigit != null)
            {
                if(szCurrentCheckDigit.equals(szCalculatedCheckDigit) == false)
                {
                    throw new Exception("Calculated CUSIP checkdigit not equal " +
                        "to supplied checkdigit. CalcChkDgt= " +
                        szCalculatedCheckDigit + " ");
                }
            }
            szRet = szCusip + szCalculatedCheckDigit;
        }
        catch(Exception ex)
        {
            System.out.println("Error in CUSIP.addChkDigit: " + ex.toString() +
                " CUSIP = " + cusip);
        }
        return szRet;
    }

    private static String generateCheckDigit(String cusip)
    {
        String szRet = null;
        String szCombinedResults = "";
        try
        {

            char[] chars = cusip.toCharArray();
            for(int k = 0; k < chars.length; k++) {
//                int pos = java.util.Arrays.binarySearch(LEGAL_VALUES, chars[k]);
                int pos = 0;
                while (chars[k] != LEGAL_VALUES[pos]) {
                    pos++;
                }
                if((k % 2) != 0)
                {
                    pos = pos * 2;
                }
                szCombinedResults += String.valueOf(pos);
            }
            int tmpVal = 0;
            chars = szCombinedResults.toCharArray();
            for(int i = 0; i < chars.length; i++) {
                tmpVal += Character.digit(chars[i], 10);
            }
            tmpVal = Math.abs(((tmpVal % 10) - 10) % 10); // Get complement
            szRet = String.valueOf(tmpVal);
        }
        catch(Exception ex)
        {
            System.err.println("Error in CUSIP.generateCheckDigit:" + ex.toString());
            System.err.println("CUSIP=" + cusip);
        }
        return szRet;
    }
    //--------------------------------------------------------------------------
    public static void main(String[] args)
    {
        try
        {

            System.out.println(CUSIP.generateCheckDigit("392690QT")); // 3
            System.out.println(CUSIP.generateCheckDigit("46127G10")); // 5
            System.out.println(CUSIP.generateCheckDigit("44929K80")); // 4
            System.out.println(CUSIP.generateCheckDigit("A1234512")); // 9
            System.out.println(CUSIP.generateCheckDigit("29372077")); // 7
            System.out.println(CUSIP.generateCheckDigit("29372023")); // 1
            System.out.println(CUSIP.generateCheckDigit("29372024")); // 9

            System.out.println(CUSIP.addChkDigit("392690QT")); // 3
            System.out.println(CUSIP.addChkDigit("293720563"));// Correct=1
            System.out.println(CUSIP.addChkDigit("46127G10")); // 5
            System.out.println(CUSIP.addChkDigit("44929K80")); // 4
            System.out.println(CUSIP.addChkDigit("29372K461"));// Correct=8
            System.out.println(CUSIP.addChkDigit("A1234512")); // 9
            System.out.println(CUSIP.addChkDigit("293720288"));// Correct=0
            System.out.println(CUSIP.addChkDigit("29372028")); // 0
            System.out.println(CUSIP.addChkDigit("29372K527"));// Correct=6
            System.out.println(CUSIP.addChkDigit("29372023")); // 1
            System.out.println(CUSIP.addChkDigit("29372024")); // 9
            System.out.println(CUSIP.addChkDigit("29372K609"));// OK
            System.out.println(CUSIP.addChkDigit("29372K518"));// OK

        }
        catch(Exception ex)
        {
            //
        }
    }
}


/*

CUSIPs

293720306
293720819
293720579
29372K575
29372K567
29372K559
293720488
293720470
293720462
293720207
293720827
293720595
293720355
293720348
293720330
29372K666
29372K658
29372K641
29372K104
29372K203
29372K302
293720173
293720165
293720157
293720603
293720777
293720538
293720108
293720835
293720611
293720447
293720439
293720421
293720801
293720769
293720520
293720405
293720785
293720553
293720231
293720249
293720256
293720843
293720744
293720546
29372k625
29372k617
29372k591
29372K500
29372K609
29372K708
293720868
293720736
293720496
293720272
293720280
293720298
293720397
293720389
293720371
293720850
293720793
293720561
29372K484
29372K476
29372K468
293720876
293720751
293720512
29372K534
29372K526
29372K518

*/