package com.esdi.util;

import javax.mail.internet.*;
import javax.mail.*;
import javax.mail.event.*;
import java.util.logging.*;
import java.util.*;

/**
 * An EmailHandler is a Handler that buffers its input in memory until
 * it is closed, then sends the formatted logs as an email to the recipients
 * specified in the constructor.  Once closed further calls to publish() will
 * be ignored.
 * <p>
 * You can call close multiple times, this will resend the email.  If the recipients,
 * sender, or server is changed the new values will be used.
 */
public class EmailHandler
extends Handler
implements ConnectionListener, TransportListener {

    protected List records = new ArrayList();
    public String
        recipients,
        sender,
        server,
        subject;

    /**
     * Create a EmailHandler that will email the log records to recipients on being
     * closed.
     * @param recipients a valid smtp address list
     * @param sender a string identifying the sender of the email
     * @param server the email server
     * @param subject the subject of the email.
     */
    public EmailHandler(String recipients, String sender, String server, String subject) {
        this.recipients = recipients;
        this.sender = sender;
        this.server = server;
        this.subject = subject;
    }

    /**
     * Buffer the record for emailing when the handler is closed
     * @param record
     */
    public void publish(LogRecord record) {
        records.add(record);
    }

    /**
     * Does nothing
     */
    public void flush() {
        // do nothing
    }

    /**
     * Send the email to the recipients.
     * @throws SecurityException
     * @throws IllegalArgumentException if there was a messaging exception
     */
    public void close() throws SecurityException {
        mailReport(getRecordString());
    }

    public Formatter getFormatter() {
        if (super.getFormatter() == null) {
            setFormatter(new SimpleFormatter());
        }
        return super.getFormatter();
    }
    /////////////////////////////////////////////////////////////////////////////////
    // INTERFACES
    /////////////////////////////////////////////////////////////////////////////////

    //
    // ConnectionListener interface  -----------------------------------
    //
    //
    public void opened(ConnectionEvent e) {
    }

    public void disconnected(ConnectionEvent e) {
    }

    public void closed(ConnectionEvent e) {
    }

    //
    // TransportListener interface -----------------------------------
    //
    public void messageDelivered(TransportEvent e) {
    }

    public void messageNotDelivered(TransportEvent e) {
        System.err.print(">>> TransportListener.messageNotDelivered().");
        System.err.println(" Invalid Addresses:");
        Address[] invalid = e.getInvalidAddresses();
        if (invalid != null) {
            for (int i = 0; i < invalid.length; i++)
                System.err.println("     " + invalid[i]);
        }
    }

    public void messagePartiallyDelivered(TransportEvent e) {
    }


    /////////////////////////////////////////////////////////////////////////////////
    // IMPLEMENTATION
    /////////////////////////////////////////////////////////////////////////////////

    protected void mailReport(String message) {
        InternetAddress[] addr = null;
        Properties props = new Properties();
        props.put("mail.smtp.user", sender);
        props.put("mail.smtp.host", server);

        try {
            addr = InternetAddress.parse(recipients);   // to
        } catch (AddressException ex) {
            throw new IllegalArgumentException(ex.toString());
        }

        Session session = Session.getInstance(props, null);
        session.setDebug(false);

        Transport trans = null;
        Message msg = new MimeMessage(session);

        try {
            trans = session.getTransport(addr[0]);
            trans.addConnectionListener(this);
            trans.addTransportListener(this);
            trans.connect();

            msg.setFrom();
            msg.setRecipients(Message.RecipientType.TO, addr);
            msg.setSubject(subject);
            msg.setSentDate(new java.util.Date());
            msg.setContent(message, "text/plain");
            trans.sendMessage(msg, addr);
            try {
                Thread.sleep(5);
            } catch (InterruptedException e) {
            }

        } catch (MessagingException ex) {
            throw new IllegalArgumentException(ex.toString());
        }
    }

    protected String getRecordString() {
        StringBuffer sb = new StringBuffer(records.size() * 100);
        for (Iterator iterator = records.iterator(); iterator.hasNext();) {
            LogRecord logRecord = (LogRecord) iterator.next();
            sb.append(getFormatter().format(logRecord));
        }
        return sb.toString();
    }
}
