package com.esdi.util;

/**
 * <p>Title: TelekursCAKeyScheme</p>
 * <p>Description: Telekurs Corporate Actions are identifed by
 * Valorin + ContentCode + MessageDate + MessageNbr + Record Type:</p>
 * <ul>
 * <li>Valorin of security: chars 3,9; eg </li>
 * <li>ContentCode pos 16,3</li>
 * <li>MessageDate = pos 19,8</li>
 * <li>MessageNbr = pos 29,2</li>
 * <li>Now the record type is hard to explain, sometimes it follows the record type on the
 * file and sometimes we add a number after the record type if there are more than 2 C.A.
 * records to be created [sic].</li>
 * </ul>
 * Because of the way Telekurs Corporate Action files work, the same instance of TelekursCAKeyScheme
 * should be used for all lines.
 *
 * <p>Copyright: Copyright (c) Emphasis Software Development 2002</p>
 * @author Martin Buis
 * @version 1.0
 */

public class TelekursCAKeyScheme
implements CAParserKeyScheme {

    public TelekursCAKeyScheme() {
    }

    /**
     * Return true if lineOrKey is the first line of a record
     * <p>
     * We can receive multiple records for the same valoren one after another.  Conceivably, we could
     * receive several 'A' lines in a row, each for a different record.  In any case, though, we
     * will always know that we're in a new record when the line's record type is 'A'.
     */
    public boolean isNewRecord(String priorKey, String lineOrKey) {
        return ! buildKey(lineOrKey).equals(priorKey)
               || lineOrKey.charAt(12) == 'A';
    }

    /**
     * Return the key made up of the valoren, content code, message date, message number and
     * record type for the line.  Assumes that records always start with an 'A' record type
     * <p>
     * These fields are only present on the first line of a record (each Telekurs CA record
     * extends over several lines).  Subsequent lines in the record have the same valoren, but a
     * different record type (13,1) with different fields so we can't extract a key from them.
     * Wee need to remember the key from the first line and return it for the subsequent lines until
     * we are no longer in the record.
     */
    public String getKey(String line) {
        return buildKey(line);
    }

    /////////////////////////////////////////////////////////////////////////////////
    // IMPLEMENTATION
    /////////////////////////////////////////////////////////////////////////////////

    private String buildKey(String line) {
        String valoren = line.substring(3,9);
        String contentCode = line.substring(16,19);
        String messageDate = line.substring(19,27);
        String messageNumber = line.substring(29,31);
        return valoren + contentCode + messageDate + messageNumber;
    }


    /////////////////////////////////////////////////////////////////////////////////
    // MAIN
    /////////////////////////////////////////////////////////////////////////////////

    public static void main(String[] args) {
        String[] lines = {
            "CH000101516*A08110200106252101                         1                      9N   JYN N N  NN6.75    420050701          USD",
            "CH000101516*BEnron Corp",
            "CH000101516*C6 3/4 % Debs /1993-1.7.05 Sr Subord -In Default-                                                                       US293561AT33",
            "CH000101516*DEnron                                                            921231                                 US 102",
            "CH000101516*E6.75 Enron 05SSDebs      0  26           USD",
            "CH000101516*F                                                                                                                           US 11 Y",
            "CH000101516*H19930701",
            "CH000101516*I                     01012XY",
            "CH000101516*A08111199307222101                                                                         20050701          USD200000000",
            "CH000101516*BEnron Corp",
            "CH000101516*C6 3/4 % Debs /1993-1.7.05 Sr Subord -In Default-",
            "CH000101516*DEnron                                                                                                   US",
            "CH000101516*E6.75 Enron 05SSDebs      0  26",
            "CH000101516*F1000        1000                                                                                                                  1000",
            "CH000101516*GLehman Brothers            Nationsbank Tx             10",
            "CH000101516*H19930701                                   99.497  4                                                                                    ",
            "CH000105544*A08130200106252101                                                                         20930715          USD",
            "CH000107234*A08130200106252101                                                                         20930715          USD" };

        TelekursCAKeyScheme s = new TelekursCAKeyScheme();
        for (int i = 0; i < lines.length; i++) {
            String line = lines[i];
            String key = s.getKey(line);
            System.out.println(key + ", " + line);
        }
    }

}