package com.gonetix.framework.dbengine;


import java.sql.*;
import java.util.*;
import javax.naming.*;
import javax.sql.*;


public final class SQLExecutor 
{
  private static final String DRIVER = "jdbc.driver";
  private static final String URL = "jdbc.url";
  private static final String USER = "jdbc.user";
  private static final String PASSWORD = "jdbc.password";
  private static final String JNDINAME = "jdbc.jndiname";  
  private static final String DEFAULT = "default";
  
  protected final static SQLExecutor handler = new SQLExecutor();
  private Connection conn = null;
  protected Hashtable stmtHolder = null;
  protected Hashtable preparedHolder = null;
  private Properties dbProp = null;

  /*
  *this is a singleton instanciation method
  */
  public synchronized static SQLExecutor getSQLExecutor()
  {
      return handler;
  }
    
  protected SQLExecutor()
  {       
      stmtHolder = new Hashtable(100);
      preparedHolder = new Hashtable(100);
  }

  public synchronized void connect(Properties prop) throws SQLException
  {
      if ( conn==null || conn.isClosed() ){
          dbProp = prop;
          conn = createConnection(dbProp);
      }
  }


  public static Connection createConnection(Properties prop) throws SQLException
  {
      Connection con = null;
      String jndiname = prop.getProperty(JNDINAME);
      if ( jndiname!=null ){            
          DataSource ds = null;  
          try{
              InitialContext context = new InitialContext();
              ds = (DataSource)context.lookup(jndiname);              
          }catch(Throwable e){
              throw new SQLException("FATAL: SQLExecutor JNDI lookup excepton.\n" + e.getMessage());
          }
          if ( ds!=null) {
              con = ds.getConnection();
          }else{
              throw new SQLException("FATAL: SQLExecutor JNDI lookup excepton, no DataSource Found.\n");
          }
      }else{
          try{
              Class.forName(prop.getProperty(DRIVER)).newInstance();
          }catch(Throwable e){
              throw new SQLException("FATAL: SQLExecutor loading driver excepton.\n" + e.getMessage());
          }
          con = DriverManager.getConnection(prop.getProperty(URL), prop.getProperty(USER), prop.getProperty(PASSWORD));
      }
      return con;
  }

  public ResultSet executeQuery(String sql) throws SQLException
  {
      ResultSet rs = null;
      Statement stmt = conn.createStatement();
      rs = stmt.executeQuery(sql);
      stmtHolder.put(rs, stmt);
      return rs;
  }
  
  public void closeResultSet(ResultSet rs)
  {
      if (rs==null) return;
      try{
          Statement stmt = (Statement)stmtHolder.remove(rs);
          rs.close();
          if (stmt!=null) stmt.close();
      }catch(Exception ex){
          ;
      }
  }

  public void query(String sql, IResultProcessor processor) throws SQLException
  {
      ResultSet rs = null;
      Statement stmt = conn.createStatement();
      rs = stmt.executeQuery(sql);
      if ( rs!=null ){
          processor.process(rs);         
      }
      stmt.close();
  }

  public void execute(String sql) throws SQLException
  {
        Statement stmt = conn.createStatement();
        int b = -1;
        ResultSet rs = null;
        boolean more = stmt.execute(sql);
        while ( more || (b=stmt.getUpdateCount())!=-1 ){            
            if ( more ){                  
                rs = stmt.getResultSet();                              
            }else if (b!=-1){
               b = -1;
            }
            more = stmt.getMoreResults();
        }
        stmt.close();
  }
  
  public void execute(String sql, IResultProcessor processor) throws SQLException
  {
        Statement stmt = conn.createStatement();
        int b = -1;
        ResultSet rs = null;
        boolean more = stmt.execute(sql);
        while ( more || (b=stmt.getUpdateCount())!=-1 ){            
            if ( more ){                  
                rs = stmt.getResultSet();
                if ( rs!=null )processor.process(rs);                
            }else if (b!=-1){                  
               processor.process(b);
               b = -1;
            }
            more = stmt.getMoreResults();
        }
        stmt.close();
  }

  public int executeUpdate(String sql) throws SQLException
  {
      Statement stmt = conn.createStatement();
      int ret = stmt.executeUpdate(sql);
      stmt.close();
      return ret;
  }

  public int[] executeBatchUpdate(String sql) throws SQLException
  {
      Statement stmt = conn.createStatement();
      stmt.addBatch(sql);
      int[] ret = stmt.executeBatch();
      stmt.close();
      return ret;
  }

  public int[] executeBatchUpdate(Collection sqls) throws SQLException
  {
      if (sqls == null || sqls.size()== 0) return (new int[0]);
      Statement stmt = conn.createStatement();
      Iterator iterator = sqls.iterator(); 
      while (iterator.hasNext()){
            stmt.addBatch((String)iterator.next());
      }      
      int[] ret = stmt.executeBatch();
      stmt.close();
      return ret;
  }

  public int[] executeBatchUpdate(String[] sqls) throws SQLException
  {
      if (sqls == null || sqls.length == 0 ) return (new int[0]);
      Statement stmt = conn.createStatement();      
      for(int i=0; i<sqls.length; i++){
            stmt.addBatch(sqls[i]);
      }      
      int[] ret = stmt.executeBatch();
      stmt.close();
      return ret;
  }


  public void prepare(String key, String sql)  throws SQLException
  {
      if ( key ==null) key = DEFAULT;
      PreparedStatement pstmt = conn.prepareStatement(sql);
      preparedHolder.put(key, pstmt);
  }

  public void prepare(String sql)  throws SQLException
  {
     prepare(null, sql);
  }
  public ResultSet executePreparedQuery(Object[] params, int[] types)  throws SQLException
  {
        return executePreparedQuery(null, params, types);
  }
  
  
  public ResultSet executePreparedQuery(String key, Object[] params, int[] types)  throws SQLException
  {
      if ( key ==null) key = DEFAULT;      
      if ( params.length != types.length ) throw new SQLException("FATAL: Parameters count and Types count are not match.");
      PreparedStatement pstmt = (PreparedStatement)preparedHolder.get(key);      
      if ( pstmt ==null ) throw new SQLException("FATAL: SQL has not been prepared yet. --> " + key);
      pstmt = prepareSatement(pstmt, params, types);
      return (pstmt.executeQuery());      
  }

  public boolean executePreparedUpdate(String key, Object[] params, int[] types)  throws SQLException
  {
      boolean ok = false;
      try{
          executePrepared(key,  params, types, null);
          ok = true;
      }catch(SQLException ex){
          ok = false;
          throw ex;
      }
      return ok;      
  }

  public boolean executePreparedUpdate(Object[] params, int[] types)  throws SQLException
  {      
      return executePreparedUpdate(null,  params, types); 
  }
  
  public void executePrepared(Object[] params, int[] types, IResultProcessor processor)  throws SQLException
  {
        executePrepared(null, params, types, processor);
  }
  
  
  public void executePrepared(String key, Object[] params, int[] types, IResultProcessor processor)  throws SQLException
  {
      if ( key ==null) key = DEFAULT;      
      if ( params.length != types.length ) throw new SQLException("FATAL: Parameters count and Types count are not match.");
      PreparedStatement pstmt = (PreparedStatement)preparedHolder.get(key);      
      if ( pstmt ==null ) throw new SQLException("FATAL: SQL has not been prepared yet. --> " + key);
      pstmt = prepareSatement(pstmt, params, types);
      int b = -1;
      ResultSet rs = null;
      boolean more = pstmt.execute();
      while ( more || (b=pstmt.getUpdateCount())!=-1 ){            
          if ( more ){                  
              rs = pstmt.getResultSet();
              if ( rs!=null && processor!= null)processor.process(rs);                
          }else if (b!=-1){                  
             if ( processor!= null) processor.process(b);
             b = -1;
          }
          more = pstmt.getMoreResults();
      }      
  }
  
  public PreparedStatement prepareSatement(PreparedStatement pstmt, SQLParameter[] params) throws SQLException
  {
  		for(int i=0; i<params.length; i++){
            if ( params[i] == null){
                throw new SQLException("FATAL: Invalid paramter.");
             }else{
                 pstmt.setObject(i+1, params[i].getObject(), params[i].getType());
             }
      }  	
  	  return pstmt;
  }
  
  public PreparedStatement prepareSatement(PreparedStatement pstmt, Object[] params, int[] types) throws SQLException
  {
  	 for(int i=0; i<types.length; i++){
            if (params[i] == null ){
                pstmt.setNull(i+1, types[i]);
            }else{
                pstmt.setObject(i+1, params[i], types[i]);
            }
      }  	
  	  return pstmt;
  }
  
  public PreparedStatement prepareSatement(String sqlStr, Object[] params, int[] types) throws SQLException
  {
  	PreparedStatement pstmt = this.conn.prepareStatement(sqlStr);  		
  	return prepareSatement(pstmt, params, types);
  }
  
  public PreparedStatement prepareSatement(String sqlStr, SQLParameter[] params) throws SQLException
  {
  	PreparedStatement pstmt = this.conn.prepareStatement(sqlStr);  		
  	return prepareSatement(pstmt, params);
  }
  
  public Statement createBatchSatement(String sqlStr) throws SQLException
  {
  	Statement stmt = conn.createStatement();
  	stmt.addBatch(sqlStr);  		
  	return stmt;
  }
  
  public void close()
  {
      try{
          if (conn!=null) conn.close();
      }catch(Exception ex){
          ;
      }
  }
  
}